@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Core;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Core vocabulary
 
-our $VERSION = '0.597';
+our $VERSION = '0.598';
 
 use 5.020;
 use Moo;
@@ -121,9 +121,9 @@ sub _traverse_keyword_schema ($class, $schema, $state) {
   }
   else {
     my $schema_info = $state->{evaluator}->_fetch_from_uri($schema->{'$schema'});
-    return E($state, 'EXCEPTION: unable to find resource %s', $schema->{'$schema'}) if not $schema_info;
+    return E($state, 'EXCEPTION: unable to find resource "%s"', $schema->{'$schema'}) if not $schema_info;
     # this cannot happen unless there are other entity types in the index
-    return E($state, 'EXCEPTION: bad reference to $schema %s: not a schema', $schema_info->{canonical_uri})
+    return E($state, 'EXCEPTION: bad reference to $schema "%s": not a schema', $schema_info->{canonical_uri})
       if $schema_info->{document}->get_entity_at_location($schema_info->{document_path}) ne 'schema';
 
     if (not is_plain_hashref($schema_info->{schema})) {
@@ -236,8 +236,8 @@ sub _eval_keyword_ref ($class, $data, $schema, $state) {
 sub _eval_keyword_recursiveRef ($class, $data, $schema, $state) {
   my $uri = Mojo::URL->new($schema->{'$recursiveRef'})->to_abs($state->{initial_schema_uri});
   my $schema_info = $state->{evaluator}->_fetch_from_uri($uri);
-  abort($state, 'EXCEPTION: unable to find resource %s', $uri) if not $schema_info;
-  abort($state, 'EXCEPTION: bad reference to %s: not a schema', $schema_info->{canonical_uri})
+  abort($state, 'EXCEPTION: unable to find resource "%s"', $uri) if not $schema_info;
+  abort($state, 'EXCEPTION: bad reference to "%s": not a schema', $schema_info->{canonical_uri})
     if $schema_info->{document}->get_entity_at_location($schema_info->{document_path}) ne 'schema';
 
   if (is_plain_hashref($schema_info->{schema})
@@ -255,8 +255,8 @@ sub _eval_keyword_recursiveRef ($class, $data, $schema, $state) {
 sub _eval_keyword_dynamicRef ($class, $data, $schema, $state) {
   my $uri = Mojo::URL->new($schema->{'$dynamicRef'})->to_abs($state->{initial_schema_uri});
   my $schema_info = $state->{evaluator}->_fetch_from_uri($uri);
-  abort($state, 'EXCEPTION: unable to find resource %s', $uri) if not $schema_info;
-  abort($state, 'EXCEPTION: bad reference to %s: not a schema', $schema_info->{canonical_uri})
+  abort($state, 'EXCEPTION: unable to find resource "%s"', $uri) if not $schema_info;
+  abort($state, 'EXCEPTION: bad reference to "%s": not a schema', $schema_info->{canonical_uri})
     if $schema_info->{document}->get_entity_at_location($schema_info->{document_path}) ne 'schema';
 
   # If the initially resolved starting point URI includes a fragment that was created by the
@@ -342,7 +342,7 @@ JSON::Schema::Modern::Vocabulary::Core - Implementation of the JSON Schema Core
 
 =head1 VERSION
 
-version 0.597
+version 0.598
 
 =head1 DESCRIPTION
 
